(function() {
    var config = window.ultima.config;

    function getPassText(pass) {
        var text = "";

        var surname = pass.surname;
        var name = pass.name;
        var patronymic = pass.patronymic;
        if (surname) {
            text = surname;
            if ((name || "").length > 0) {
                text += " " + (name || "").substr(0, 1) + ".";
                if ((patronymic || "").length > 0) {
                    text += " " + (patronymic || "").substr(0, 1) + ".";
                }
            }
        } else {
            if (name || patronymic) {
                text = (name || "?") + " " + (patronymic || "");
            } else {
                text = "<нет ФИО>";
            }
        }
        return text;
    }

    $(".ultima-reports-generic-pass").each(function() {
        var valueInput = $(this).find(".ultima-reports-generic-pass-value");
        var hintInput = $(this).find(".ultima-reports-generic-pass-hint");
        var multipleChoice = $(this).find(".ultima-reports-generic-pass-multiple-choice").val() == "true";
        var skipDeleted = $(this).find(".ultima-reports-generic-pass-skip-deleted").val() == "true";
        var passTable = $(this).find(".ultima-reports-generic-pass-table");
        var dialogContent = $(this).find(".ultima-reports-generic-pass-dialog");
        var editButton = $(this).find(".ultima-reports-generic-pass-edit");

        function updateValue() {
            var ids = [];
            var hint = [];
            passTable.find("tr").each(function() {
                var pass = $(this).data("pass");
                ids.push(pass.id);
                hint.push(pass)
            });
            valueInput.val(ids.join(","));
            hintInput.val(JSON.stringify(hint));
        }

        hintInput.on("change", function() {
            var value = $(this).val();
            if (value == "") {
                passTable.find("tbody").empty();
            } else {
                try {
                    var list = JSON.parse(value);
                    console.log(list);
                    passTable.find("tbody").empty();
                    list.forEach(function (thisPass) {
                        passTable.show();
                        console.log("appending line");
                        passTable.find("tbody").append($("<tr>").data("pass", thisPass).append([
                            $("<td>").text("#" + thisPass.id),
                            $("<td>").text(getPassText(thisPass)),
                            $("<td>").text("Карта: " + (thisPass.cardNumber || "?")),
                            $("<td>").append(
                                $("<a class='ultima-reports-generic-pass-remove'>").text("[удалить]").click(function (e) {
                                    var tbody = $(this).parent().parent().parent();
                                    $(this).parent().parent().remove();
                                    if (tbody.children().length == 0) {
                                        passTable.hide();
                                    }
                                    editButton.attr("disabled", null).attr("title", "Нажмите для выбора пропуска");
                                    updateValue();
                                    e.preventDefault();
                                })
                            )
                        ]));
                    });
                    if (!multipleChoice && list.length > 0) {
                        editButton.attr("disabled", "disabled").attr("title", "Для этого отчёта можно выбрать только один пропуск");
                    }
                    updateValue();
                } catch (e) {
                    console.log(e);
                }
            }
        });

        editButton.text(multipleChoice ? "Добавить пропуск" : "Выбрать пропуск").attr("title", "Нажмите для выбора пропуска");
        editButton.click(function(e) {
            dialogContent.dialog({
                top: {
                    title: 'Выбрать пропуск',
                    button: 'close'
                },
                bottom: {
                    buttons: [
                        {
                            text: 'Найти',
                            action: function() {
                                var dialogTable = dialogContent.find(".ultima-reports-generic-pass-dialog-table");
                                var dialogProgress = dialogContent.find(".ultima-reports-generic-pass-dialog-progress");
                                dialogProgress.show();
                                dialogTable.hide();
                                new ultima.AjaxRequest({
                                    type: "POST",
                                    url: "/api/v1/neyross/pass/search",
                                    contentType: "application/json",
                                    data: JSON.stringify({
                                        surname: dialogContent.find(".ultima-reports-generic-pass-surname").val(),
                                        name: dialogContent.find(".ultima-reports-generic-pass-name").val(),
                                        patronymic: dialogContent.find(".ultima-reports-generic-pass-patronymic").val(),
                                        cardNumber: dialogContent.find(".ultima-reports-generic-pass-cardnumber").val(),
                                        pincode: dialogContent.find(".ultima-reports-generic-pass-pincode").val(),
                                        skipDeleted: skipDeleted
                                    })
                                }).done(function(results) {
                                    var tableBody = dialogTable.find("tbody");
                                    tableBody.empty();
                                    results.forEach(function(pass) {
                                        var thisPass = pass;
                                        var row = $("<tr>").append([
                                            $("<td>").text(pass.cardNumber),
                                            $("<td>").text(pass.pincode),
                                            $("<td>").text(pass.surname || ""),
                                            $("<td>").text(pass.name || ""),
                                            $("<td>").text(pass.patronymic || "")
                                        ]);
                                        row.click(function() {
                                            passTable.show();
                                            passTable.find("tbody").append($("<tr>").data("pass", thisPass).append([
                                                $("<td>").text("#" + thisPass.id),
                                                $("<td>").text(getPassText(thisPass)),
                                                $("<td>").text("Карта: " + (thisPass.cardNumber || "?")),
                                                $("<td>").append(
                                                    $("<a class='ultima-reports-generic-pass-remove'>").text("[удалить]").click(function(e) {
                                                        var tbody = $(this).parent().parent().parent();
                                                        $(this).parent().parent().remove();
                                                        if (tbody.children().length == 0) {
                                                            passTable.hide();
                                                        }
                                                        editButton.attr("disabled", null).attr("title", "Нажмите для выбора пропуска");
                                                        updateValue();
                                                        e.preventDefault();
                                                    })
                                                )
                                            ]));
                                            if (!multipleChoice) {
                                                editButton.attr("disabled", "disabled").attr("title", "Для этого отчёта можно выбрать только один пропуск");
                                            }
                                            updateValue();
                                            dialogContent.dialog("close");
                                        });
                                        tableBody.append(row);
                                    });
                                    // fill table
                                    dialogProgress.hide();
                                    dialogTable.show();
                                }).fail(function(e) {

                                });
                            }
                        }
                    ]
                }
            });
            e.preventDefault();
            return false;
        });
    });
})();