package extensions.reports

import play.api.libs.Files.TemporaryFile
import play.api.mvc._
import play.twirl.api.Html
import utils.reports.{ReverseRouting, Context, PartialReverseRouting}

import scala.collection.mutable

trait GenericFormField {
  def key(): String
  def title(): String
  def render(): Html
  def parse(value: String): AnyRef
}

trait GenericFormSupport {
  protected val formFields = mutable.Map[String, (GenericFormField, Int)]()

  protected var reverseRouting: ReverseRouting

  protected def registerField(field: GenericFormField): Unit = {
    formFields += (field.key() -> (field, formFields.size))
  }

  protected def getFields: Seq[GenericFormField] = {
    formFields.values.toList.sortBy(_._2).map(_._1).toSeq
  }

  /*
  protected def initForm(parameters: Iterable[(AnyRef, AnyRef)]) = {
    parameters.foreach {
      case (fieldType, data) =>
        fieldFactory.get(fieldType) match {
          case Some(factory) => {
            val field = factory(data)
            formFields += (field.key() -> field)
          }
          case _ => // FIXME
        }
    }
  }
   */

  def renderFormAction: Action[AnyContent] = Action { request =>
    val routing = reverseRouting.forDataSource(Context.getDataSourceId)
    Results.Ok(views.html.reports.generic.form2(getFields, routing))
  }

  def processForm(request: Request[MultipartFormData[TemporaryFile]]): AnyRef = {
    val map = mutable.Map[String, AnyRef]()

    request.body.dataParts.foreach {
      case (key, value) if formFields.contains(key) =>
        if (value.size == 1) {
          map += (key -> parse(key, value.head))
        } else if (value.size > 1) {
          map += (key -> value.map(s => parse(key, s)))
        }
      case _ => // do nothing
    }

    map.toMap
  }

  private def parse(key: String, value: String): AnyRef = {
    formFields.get(key).get._1.parse(value)
  }
}
