package extensions.reports

import java.io.{File, InputStream}

import com.typesafe.config.Config
import play.api.libs.Files.TemporaryFile
import play.api.mvc._
import play.core.j.JavaHelpers
import play.mvc.{Controller => JavaController, Http}

trait ReportDefinitionLoader {
  def getResourceAsStream(name: String): Option[InputStream]
  def getDefinitionDirectory: String
  def getClassLoader: ClassLoader
}

abstract class ReportDefinition {
  def init(config: Config, loader: ReportDefinitionLoader)

  def key: String

  def title: String

  def description: String

  def renderFormAction: Action[AnyContent] = Action { request =>
    val oldContext = Http.Context.current.get()
    try {
      Http.Context.current.set(JavaHelpers.createJavaContext(request))
      renderForm(JavaController.request(), JavaController.response()).asScala()
    } finally {
      Http.Context.current.set(oldContext)
    }
  }

  def processForm(request: Request[MultipartFormData[TemporaryFile]]): AnyRef = {
    val oldContext = Http.Context.current.get()
    try {
      Http.Context.current.set(JavaHelpers.createJavaContext(request))
      processForm(JavaController.request())
    } finally {
      Http.Context.current.set(oldContext)
    }
  }

  def generateReport(format: String, form: AnyRef, dataSource: javax.sql.DataSource, to: File): File

  def renderForm(request: Http.Request, response: Http.Response): play.mvc.Result = {
    throw new NotImplementedError(s"renderForm() not implemented for class ${this.getClass.getName}")
  }

  def processForm(request: Http.Request): AnyRef = {
    throw new NotImplementedError(s"processForm() not implemented for class ${this.getClass.getName}")
  }

  def customAction(path: String): Action[AnyContent] = Action {
    Results.NotFound
  }

  def getProperty(key: String): Option[String] = {
    None
  }
}
