package extensions.reports.generic

import java.sql.Timestamp

import net.sf.jasperreports.engine.JRParameter
import org.joda.time.{LocalTime, DateTime}
import org.joda.time.format.{DateTimeFormatter, DateTimeFormat}
import play.twirl.api.Html
import utils.reports.FieldProperties

class DateTimeFormField()
  extends AbstractJasperFormField {

  import DateTimeFormField._

  private var pickDate: Boolean = true
  private var pickTime: Boolean = true
  private var jsFormat: String = "dd.MM.yyyy hh:mm:ss"
  private var javaFormat: String = "dd.MM.YYYY HH:mm:ss"
  private var timeOfDay: String = "00:00:00.000"

  def this(key: String, title: String) = {
    this()
    keyValue = key
    titleValue = title
  }

  override def init(jrParameter: JRParameter): Unit = {
    super.init(jrParameter)

    pickDate = getParameterProperty(jrParameter, FieldProperties.PICK_DATE).map(_.toBoolean).getOrElse(true)
    pickTime = getParameterProperty(jrParameter, FieldProperties.PICK_TIME).map(_.toBoolean).getOrElse(true)
    timeOfDay = getParameterProperty(jrParameter, FieldProperties.TIME_OF_DAY).getOrElse("00:00:00.000")
    jsFormat = getParameterProperty(jrParameter, FieldProperties.JS_FORMAT).getOrElse(
      if (pickDate && pickTime) {
        "dd.MM.yyyy hh:mm:ss"
      } else if (pickDate) {
        "dd.MM.yyyy"
      } else if (pickTime) {
        "hh:mm"
      } else {
        ""
      }
    )
    javaFormat = getParameterProperty(jrParameter, FieldProperties.JAVA_FORMAT).getOrElse(
      if (pickDate && pickTime) {
        "dd.MM.YYYY HH:mm:ss"
      } else if (pickDate) {
        "dd.MM.YYYY"
      } else if (pickTime) {
        "HH:mm"
      } else {
        ""
      }
    )
  }

  private def formatter(): DateTimeFormatter = {
    DateTimeFormat.forPattern(javaFormat)
  }

  override def render(): Html = {
    views.html.reports.generic.datetime(key(), DateTime.now().toString(formatter()), pickDate, pickTime, jsFormat)
  }

  override def parse(value: String): AnyRef = {
    var offset: Long = 0
    if (!pickTime) {
      offset = LocalTime.parse(timeOfDay, timeOfDayFormatter).getMillisOfDay
    }
    new Timestamp(formatter().parseDateTime(value).getMillis + offset)
  }
}

object DateTimeFormField {
  val timeOfDayFormatter = DateTimeFormat.forPattern("HH:mm:ss.SSS")
}
