package extensions.reports.generic

import models.reports.generic.{SQLChoiceOptionComponent, SQLChoiceOption}
import net.sf.jasperreports.engine.JRParameter
import play.api.Logger
import play.api.libs.json.Json
import play.api.mvc.Results
import play.twirl.api.Html
import utils.reports.{FieldProperties, Context}

class SQLChoiceFormField extends AbstractJasperFormField {
  private var isMultiple : Boolean = false
  private var sqlQuery : String = null
  private var nullName : String = null

  private val sqlOptions = new SQLChoiceOptionComponent()

  private val logger = Logger(getClass)

  override def init(jrParameter: JRParameter): Unit = {
    super.init(jrParameter)
    isMultiple = getParameterProperty(jrParameter, FieldProperties.MULTIPLE).map(_.toBoolean).getOrElse(false)
    sqlQuery = getParameterProperty(jrParameter, FieldProperties.SQL_QUERY).orNull
    nullName = getParameterProperty(jrParameter, FieldProperties.NULL_NAME).getOrElse("NULL")
  }

  override def render(): Html = {
    val dataSource = Context.getDataSource

    var options: Iterable[SQLChoiceOption] = List()

    implicit val connection = dataSource.getConnection
    try {
      options = sqlOptions.query(sqlQuery).map(option => {
        if (option.name == null) {
          SQLChoiceOption(option.id, nullName)
        } else {
          option
        }
      })
    } catch {
      case t: Throwable =>
        logger.error("failed to get SQL options; error:", t)
    } finally {
      connection.close()
    }

    views.html.reports.generic.sqlchoice(key(), options, isMultiple)
  }

  override def parse(value: String): AnyRef = {
    value
  }
}
